unit Lauta;
{
  Komponentti, joka tallettaa ruutuja, joilla on koordinaatti.
  Perusruuduissa koordinaatit eivt ole nkyviss.
  Jos laudan ruutuihin vaikuttavia asetuksia muuttaa, hyppvt
  ruudut kotiasentoonsa.

  13.10.1997/Vesa Lappalainen 

     Origo
     o-------------------------
     |       |       |        |
     |       |       |        |
     | (0,2) | (1,2) | (2,2)  |
     --------------------------
     |       |       |        |
     |       |       |        |
     | (0,1) | (1,1) | (2,1)  |
     --------------------------
     |       |       |        |
     |       |       |        |
     | (0,0) | (1,0) | (2,0)  |
     --------------------------


}

interface

uses
  SysUtils, Classes, Types,
{$ifdef CLX}
  QGraphics, QControls, QForms, QDialogs, QExtCtrls,
{$else}
  Windows, Messages,
  Graphics, Controls, Forms, Dialogs, ExtCtrls,
{$endif}
  Ruutu;

type TProcRuutu      = procedure (r:TRuutu);
type TMethRuutu      = procedure (r:TRuutu) of object;

type
  TLauta = class(TPanel)
  private
    FRuudunLeveys : integer;
    FRuudunKorkeus : integer;
    FOrigoX : integer;
    FOrigoY : integer;
    FRuutujaY : integer;
  protected
  public
    constructor Create(AOwner : TComponent); override;
    procedure SetRuudunLeveys(w:integer);                virtual;
    procedure SetRuudunKorkeus(h:integer);               virtual;
    procedure SetOrigoX(x:integer);                      virtual;
    procedure SetOrigoY(y:integer);                      virtual;
    procedure SetRuutujaY(ny:integer);                   virtual;
    function  MissaKoti(x,y:integer):TPoint;             virtual;
    procedure ForEveryRuutu(proc:TProcRuutu);            virtual;
    procedure ForEveryRuutuM(proc:TMethRuutu);           virtual;
    procedure Kotiuta;                                   virtual;
  published
    property RuudunLeveys : integer read FRuudunLeveys write SetRuudunLeveys;
    property RuudunKorkeus : integer read FRuudunKorkeus write SetRuudunKorkeus;
    property OrigoX : integer read FOrigoX write SetOrigoX;
    property OrigoY : integer read FOrigoY write SetOrigoY;
    property RuutujaY : integer read FRuutujaY write SetRuutujaY;
  end;

procedure Register;

implementation

constructor TLauta.Create(AOwner : TComponent);
begin
  inherited;
  FRuudunKorkeus := 30;
  FRuudunLeveys  := 30;
  FOrigoX        := 20;
  FOrigoY        := 20;
  Kotiuta;
end;

procedure TLauta.SetRuudunLeveys(w:integer);
begin
  FRuudunLeveys := w;
  Kotiuta;
end;

procedure TLauta.SetRuudunKorkeus(h:integer);
begin
  FRuudunKorkeus := h;
  Kotiuta;
end;

procedure TLauta.SetOrigoX(x:integer);
begin
  FOrigoX := x;
  Kotiuta;
end;

procedure TLauta.SetOrigoY(y:integer);
begin
  FOrigoY := y;
  Kotiuta;
end;

procedure TLauta.SetRuutujaY(ny:integer);
begin
  FRuutujaY := ny;
  Kotiuta;
end;

function TLauta.MissaKoti(x,y:integer):TPoint;  
begin
  Result.x := OrigoX + x * RuudunLeveys;
  Result.y := OrigoY + (RuutujaY - y - 1) * RuudunKorkeus;
end;


procedure TLauta.ForEveryRuutu(proc:TProcRuutu);
var i:integer; c : TControl;
begin
  for i:=0 to ControlCount-1 do begin
    c := Controls[i];
    if not ( c is TRuutu ) then continue;
    proc(c as TRuutu);
  end;
end;

procedure TLauta.ForEveryRuutuM(proc:TMethRuutu);
var i:integer; c : TControl;
begin
  for i:=ControlCount-1 downto 0 do begin
    c := Controls[i];
    if not ( c is TRuutu ) then continue;
    proc(c as TRuutu);
  end;
end;

procedure Kotiin(r:TRuutu);
begin
  r.UpdateRuutu;
end;

procedure TLauta.Kotiuta;
begin
  ForEveryRuutu(Kotiin);
end;


procedure Register;
begin
  RegisterComponents('TicTac', [TLauta]);
end;

end.
